//-------------------------------------------------------------------------------------------------------
// VST Plug-Ins SDK
// Version 2.4		$Date: 2006/11/13 09:08:27 $
//
// Category     : VST 2.x SDK Samples
// Filename     : filters.cpp
// Created by   : Tom Erbe
// Description  : Collection of simple filters
//
// � 2012, Tom Erbe, All Rights Reserved
//-------------------------------------------------------------------------------------------------------

#ifndef __filters__
#define __filters__

#include "public.sdk/source/vst2.x/audioeffectx.h"
//#include "Oscil.h"

enum
{
	// Global
	kNumPrograms = 16,
	// Parameters Tags
	kType = 0,
	kFreq,
	kReson,

	kNumParams
};

enum
{
	kType1PoleLP = 0,
	kType1PoleHP,
	kType1stOrdAP,
	kType1stOrdLP,
	kType1stOrdHP,
	kType2ndOrdAP,
	kType2ndOrdBR,
	kType2ndOrdBP,
	kTypeBQBWLP,
	kTypeBQBWHP,
	kTypeSVLP,
	kTypeSVHP,
	kTypeSVBP,
	kTypeSVBR,
	kTypeBypass
};

class Filters;

//------------------------------------------------------------------------
class FiltersProgram
{
friend class Filters;
public:
	FiltersProgram ();
	~FiltersProgram () {}

private:
	float preParameter[kNumParams];
	char name[24];
};



//------------------------------------------------------------------------
class Filters : public AudioEffectX
{
public:
	Filters (audioMasterCallback audioMaster);
	~Filters ();

	virtual void processReplacing (float** inputs, float** outputs, VstInt32 sampleFrames);

	virtual void setProgram (VstInt32 program);
	virtual void setProgramName (char* name);
	virtual void getProgramName (char* name);
	virtual bool getProgramNameIndexed (VstInt32 category, VstInt32 index, char* text);
	
	virtual void setParameter (VstInt32 index, float value);
	virtual float getParameter (VstInt32 index);
	virtual void getParameterLabel (VstInt32 index, char* label);
	virtual void getParameterDisplay (VstInt32 index, char* text);
	virtual void getParameterName (VstInt32 index, char* text);

	virtual void resume ();

	virtual bool getEffectName (char* name);
	virtual bool getVendorString (char* text);
	virtual bool getProductString (char* text);
	virtual VstInt32 getVendorVersion () { return 1000; }
	
	virtual VstPlugCategory getPlugCategory () { return kPlugCategEffect; }

protected:

	FiltersProgram* programs;
	float twoPi;
	float Pi;
	float halfPi;
	float oldLinFreq;
	float oldReson;
	long oldType;
	// fliter coeffiecient
	double c, d;
	// filter coefficients for biquad
	double bqA1;
	double bqA2;
	double bqA3;
	double bqB1;
	double bqB2;
	// state variables for filters
	double opY1L;
	double opY1R;
	
	double foX1L;
	double foX1R;
	double foY1L;
	double foY1R;
	
	double soX2L;
	double soX1L;
	double soY0L;
	double soY2L;
	double soY1L;
	double soX2R;
	double soX1R;
	double soY0R;
	double soY2R;
	double soY1R;
	
	double bqX2L;
	double bqX1L;
	double bqY2L;
	double bqY1L;
	double bqY0L;
	double bqX2R;
	double bqX1R;
	double bqY2R;
	double bqY1R;
	double bqY0R;
	
	double svLPL;
	double svHPL;
	double svBPL;
	double svBRL;
	double svLPR;
	double svHPR;
	double svBPR;
	double svBRR;
	
	
	float *sineTable;
	float *squareTable;
	float *rampTable;
	float *triTable;
	float guiParameter[kNumParams];
	float dspParameter[kNumParams];
	long midiBendAmount;
	long curSample;
//	Oscil *osc;	
};

#endif
